package genetika;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JTextArea;
import javax.swing.text.BadLocationException;

import model.Konfigurace;
import model.PocitacCA;
import model.Pravidlo;
import rozhrani.Mrizka;

/**
 * Tda pro provdn genetickho algoritmu a pro prbn pehrvn vslednho een.
 * Genetick algoritmus vytv poten nhodnou populaci a provd vvoj dalch generac.
 */
public class GenetickyAlgoritmus {
	
	private List<Genotyp> populace;
	private List<Genotyp> populaceProVyber;
	private JButton buttonZastavit;
	private boolean stop = false;
	private double pravdepodobnostKrizeniVUzluFunkce = 0.00;
	private boolean novaGenerace = true;
	
	public GenetickyAlgoritmus(JButton buttonZastavit) {
		this.setButtonZastavit(buttonZastavit);
        buttonZastavit.addActionListener(new ActionListener() {		// zastav sputn genetick algoritmus
			public void actionPerformed(ActionEvent arg0) {
				System.out.println("Algoritmus bude zastaven. Pokejte, prosm...");
				stop = true;
			}
		});
	}
	
	public GenetickyAlgoritmus() {
	}
	
	/**
	 * Metoda provdjc cel genetick algoritmus. Po vytvoen inicializan populace (v zvislosti na nastaven decimace)
	 * kadho jedince ohodnot, nae me bt turnajovou selekc vybrn k reprodukci, ken nebo mutaci a tm uritm zpsobem
	 * do dal generace.
	 */
	public void provadejGenetickyAlgoritmus(Mrizka mrizka, JTextArea textarea, int pocetGeneraci, int velikostPocatecniPopulace, int maximalniInicializacniHloubkaStromu, int maximalniHloubkaStromuPoKrizeni, double krizeniVUzluFunkce, double reprodukce, double krizeni, double mutace, boolean zachovavatNejzdatnejsihoJedince, boolean decimace, int pocetAutomatuVRade, int pocetRad, int velikostOkoli, int pocetKonfiguraciZaBehuAlgoritmu, int pocetKonfiguraciProUrceniVysledku, boolean prehravaniPohybu, int prehravaniPoXGeneraci, int metodaInicializace) {
		
		Date cas = new Date();
		SimpleDateFormat formatCasu = new SimpleDateFormat ("hh:mm:ss");
		System.out.println("Genetick algoritmus byl zahjen v ase " + formatCasu.format(cas) + ".");
		
		stop = false;
		pravdepodobnostKrizeniVUzluFunkce = krizeniVUzluFunkce;

		List<Gen> mnozinaTerminalu = new ArrayList<Gen>();
		List<Gen> mnozinaFunkci = new ArrayList<Gen>();
		  
		Gen s = new Terminal("S");
		Gen z1 = new Terminal("Z1");
		Gen z2 = new Terminal("Z2");
		Gen z3 = new Terminal("Z3");
		Gen v1 = new Terminal("V1");
		Gen v2 = new Terminal("V2");
		Gen v3 = new Terminal("V3");
		 
		mnozinaTerminalu.add(s);
		mnozinaTerminalu.add(z1);
		mnozinaTerminalu.add(z2);
		mnozinaTerminalu.add(z3);
		mnozinaTerminalu.add(v1);
		mnozinaTerminalu.add(v2);
		mnozinaTerminalu.add(v3);
		  
		Gen and = new Gen("AND", 2);
		Gen nand = new Gen("NAND", 2);
		Gen or = new Gen("OR", 2);
		Gen nor = new Gen("NOR", 2);
		Gen xor = new Gen("XOR", 2);
		Gen xnor = new Gen("XNOR", 2);
		Gen not = new Gen("NOT", 1);
		Gen ifg = new Gen("IF", 3);
		  
		mnozinaFunkci.add(and);
		mnozinaFunkci.add(nand);
		mnozinaFunkci.add(or);
		mnozinaFunkci.add(nor);
		mnozinaFunkci.add(xor);
		mnozinaFunkci.add(xnor);
		mnozinaFunkci.add(not);
		mnozinaFunkci.add(ifg);
		
		populace = new ArrayList<Genotyp>(velikostPocatecniPopulace);
		
		int pocetReprodukci = (int) (reprodukce * velikostPocatecniPopulace);
		int pocetKrizeni = (int) (krizeni * velikostPocatecniPopulace) / 2;		// /2 - po 1 krizeni vzniknou 2 jedinci
		int pocetMutaci = (int) (mutace * velikostPocatecniPopulace);
		
		while ((pocetReprodukci + (pocetKrizeni * 2) + pocetMutaci) < velikostPocatecniPopulace) {
			pocetReprodukci++;
		}
		
		if (zachovavatNejzdatnejsihoJedince) pocetReprodukci = pocetReprodukci - 1;
		
		// DECIMACE - parametry:
		int generaceDecimace = 10;
		int pocetJedincuPredDecimaci = 10*velikostPocatecniPopulace;
		
		// DECIMACE - inicializan populace
		if (decimace) {
			populace = new ArrayList<Genotyp>(pocetJedincuPredDecimaci);
			
			pocetReprodukci = (int) (reprodukce * pocetJedincuPredDecimaci);
			pocetKrizeni = (int) (krizeni * pocetJedincuPredDecimaci) / 2;		// /2 - po 1 krizeni vzniknou 2 jedinci
			pocetMutaci = (int) (mutace * pocetJedincuPredDecimaci);
			
			while ((pocetReprodukci + (pocetKrizeni * 2) + pocetMutaci) < pocetJedincuPredDecimaci) {
				pocetReprodukci++;
			}
			
			if (zachovavatNejzdatnejsihoJedince) pocetReprodukci = pocetReprodukci - 1;
			
			vytvorPocatecniPopulaci(mnozinaTerminalu, mnozinaFunkci, maximalniInicializacniHloubkaStromu, pocetJedincuPredDecimaci, metodaInicializace);
		
		} else {
			vytvorPocatecniPopulaci(mnozinaTerminalu, mnozinaFunkci, maximalniInicializacniHloubkaStromu, velikostPocatecniPopulace, metodaInicializace);
		}
		
		Genotyp nejlepsiGenotyp = new Genotyp(populace.get(0));
		
		
		for (int i = 0; i < pocetGeneraci; i++) {		// pro kazdou generaci
			novaGenerace = true;						// pznak nov generace (kvli selekn metod "vbr podle poad")
			
			if (stop) {
				System.out.println("Genetick algoritmus byl ukonen.");
				break;
			}

			Genotyp nejlepsiGenotypVGeneraci = new Genotyp(populace.get(0));
			
			// hodnotc funkce, kadmu genotypu piad zdatnost
			for (int j = 0; j < populace.size(); j++) {	// pro kadho jedince v populaci
				
				populace.get(j).setZdatnost(0);
				
				/** pevod genotypu na objekt Pravidlo): */
				Pravidlo p = new Pravidlo(velikostOkoli);
				for (int m = 0; m < p.getTabulkaOkoli().size(); m++) {
					Konfigurace konf = new Konfigurace(velikostOkoli);
					konf.setPosloupnost(p.getTabulkaOkoli().get(m));
					p.zapisVysledek(populace.get(j).getKoren().provedSeBezZobrazeni(konf));
				}
				
				for (int ij = 0; ij < pocetKonfiguraciZaBehuAlgoritmu; ij++) {

					/** TADY se mus pustit Pravidlo na Konfiguraci k pro vytvoen urenho potu dk
					 	+ mus se spotat pevldajc barva
					 	+ mus se pidat 1 bod zdatnosti, pokud posledn dek obsahuje jen pevldajc barvu */

					PocitacCA pocitac = new PocitacCA();
					List<Konfigurace> konfList;
					konfList = new ArrayList<Konfigurace>();
					konfList.add(new Konfigurace(pocetAutomatuVRade));
					konfList.get(0).vygenerujNahodnouKonfiguraci();
					boolean prevladajiciBarva = pocitac.zjistiPrevladajiciBarvu(konfList.get(0));
					for (int in = 0; in < pocetRad; in++) {
						konfList.add(new Konfigurace(pocetAutomatuVRade));
						konfList.set(in + 1, pocitac.vypocitejDalsiRadku(konfList.get(in), p));
					}
					
					if (pocitac.zjistiBarvuRadky(konfList.get(konfList.size() - 1)) == 0 && prevladajiciBarva == false) {
						populace.get(j).setZdatnost(populace.get(j).getZdatnost() + 1);
					} else if (pocitac.zjistiBarvuRadky(konfList.get(konfList.size() - 1)) == 1 && prevladajiciBarva == true) {
						populace.get(j).setZdatnost(populace.get(j).getZdatnost() + 1);
					}
				}
				
				if (populace.get(j).getZdatnost() > nejlepsiGenotyp.getZdatnost()) {
					nejlepsiGenotyp = new Genotyp(populace.get(j));
				}
				
				if (populace.get(j).getZdatnost() > nejlepsiGenotypVGeneraci.getZdatnost()) { // uchovame jedince s nejvyssi zdatnosti v generaci
					nejlepsiGenotypVGeneraci = new Genotyp(populace.get(j));
				}
			}
			
			cas = new Date();
			System.out.println("Nejlep zdatnost v generaci " + i + ": " + nejlepsiGenotypVGeneraci.getZdatnost() + ". as dokonen tto generace: " + formatCasu.format(cas) + ".");
			
			//System.out.println("Barva posledn dky je: " + pocitac.zjistiBarvuRadky(konfList.get(konfList.size() - 1)));
		
			
			// DECIMACE - provedeni
			if (decimace && (i == (generaceDecimace - 1))) {				// decimace v n-t generaci

				// seadme celou populaci podle zdatnosti a vybereme jen prvnch x, kde x je velikost inicializan populace.

				quicksort(populace, 0, populace.size());
				
				List<Genotyp> populacePoDecimaci = new ArrayList<Genotyp>(velikostPocatecniPopulace);
				for (int k = 0; k < velikostPocatecniPopulace; k++) {
					populacePoDecimaci.add(new Genotyp(populace.get(k)));
				}
				
				pocetReprodukci = (int) (reprodukce * velikostPocatecniPopulace);
				pocetKrizeni = (int) (krizeni * velikostPocatecniPopulace) / 2;		// /2 - po 1 krizeni vzniknou 2 jedinci
				pocetMutaci = (int) (mutace * velikostPocatecniPopulace);
				
				while ((pocetReprodukci + (pocetKrizeni * 2) + pocetMutaci) < velikostPocatecniPopulace) {
					pocetReprodukci++;
				}
				
				if (zachovavatNejzdatnejsihoJedince) pocetReprodukci = pocetReprodukci - 1;
				
				populace = populacePoDecimaci;
			}
			
			
			/** ukonen innosti a zpesnn a vpis vsledku: */
			if ((i == pocetGeneraci - 1)) {
				
				// vykreslen nejlepho een do mky
				
				/** pehrn nejzdatnjho genotypu:*/
				/** pevod genotypu na objekt Pravidlo): */
				Pravidlo p = new Pravidlo(velikostOkoli);
				for (int m = 0; m < p.getTabulkaOkoli().size(); m++) {
					Konfigurace konf = new Konfigurace(velikostOkoli);
					konf.setPosloupnost(p.getTabulkaOkoli().get(m));
					p.zapisVysledek(populace.get(i).getKoren().provedSeBezZobrazeni(konf));
				}
				
				PocitacCA pocitac = new PocitacCA();
				List<Konfigurace> konfList;
				konfList = new ArrayList<Konfigurace>();
				konfList.add(new Konfigurace(pocetAutomatuVRade));
				konfList.get(0).vygenerujNahodnouKonfiguraci();
				for (int in = 0; in < pocetRad; in++) {
					konfList.add(new Konfigurace(pocetAutomatuVRade));
					konfList.set(in + 1, pocitac.vypocitejDalsiRadku(konfList.get(in), p));
				}
				
				mrizka.setKonfigurace(konfList);
				mrizka.repaint();
				
				textarea.setText(nejlepsiGenotyp.getKoren().vypis());
				System.out.println("Vsledkem bhu genetickho algoritmu je aktuln zobrazen een s celkovou zdatnost " + nejlepsiGenotyp.getZdatnost() + " (ze " + pocetKonfiguraciZaBehuAlgoritmu + " nhodnch konfigurac).");
				System.out.println("Probh zpesnn vsledn spnosti pomoc testu na " + pocetKonfiguraciProUrceniVysledku + " novch nhodnch konfigurac. Pokejte, prosm...");
				nejlepsiGenotyp.getKoren().setNejvyssiHloubka(nejlepsiGenotyp.getKoren());
				
				int zdatnost = 0;
				
				for (int ij = 0; ij < pocetKonfiguraciProUrceniVysledku; ij++) {

					/** tady se mus spustit Pravidlo na Konfiguraci k pro vytvoen urenho potu dk
					 	+ mus se spotat pevldajc barva
					 	+ mus se pidat 1 bod zdatnosti, pokud posledn dek obsahuje jen pevldajc barvu */

					pocitac = new PocitacCA();
					konfList = new ArrayList<Konfigurace>();
					konfList.add(new Konfigurace(pocetAutomatuVRade));
					konfList.get(0).vygenerujNahodnouKonfiguraci();
					boolean prevladajiciBarva = pocitac.zjistiPrevladajiciBarvu(konfList.get(0));
					boolean ukonciPredcasne = false;
					for (int in = 0; in < pocetRad; in++) {
						if (konfList.get(in).getPorovnani() == 20 || konfList.get(in).getPorovnani() == 21) {	// barva stejn jako u pedchoz dky - dosaeno konce
							ukonciPredcasne = true;
							if (prevladajiciBarva == true && konfList.get(in).getPorovnani() == 21) {
								// bl vyla a bl je pevldajc
								zdatnost = zdatnost + 1;
							} else if (prevladajiciBarva == false && konfList.get(in).getPorovnani() == 20) {
								// ern vyla a ern je pevldajc
								zdatnost = zdatnost + 1;
							}
							break;
						} else if (konfList.get(in).getPorovnani() == 3) { // barva rozdln oproti pedchoz dce -> proukovn
							ukonciPredcasne = true;
							break;
						}
						konfList.add(new Konfigurace(pocetAutomatuVRade));
						konfList.set(in + 1, pocitac.vypocitejDalsiRadku(konfList.get(in), p));
					}
					
					if (! ukonciPredcasne) {
						if (pocitac.zjistiBarvuRadky(konfList.get(konfList.size() - 1)) == 0 && prevladajiciBarva == false) {
							zdatnost = zdatnost + 1;
						} else if (pocitac.zjistiBarvuRadky(konfList.get(konfList.size() - 1)) == 1 && prevladajiciBarva == true) {
							zdatnost = zdatnost + 1;
						}
					}
				}
				
				System.out.println("zdatnost: " + zdatnost);
				double vysledek = ((double)zdatnost / (double)pocetKonfiguraciProUrceniVysledku) * 100;
				
				System.out.println("Vpoet byl ukonen.");
				System.out.println("VSLEDN SP̊NOST JE " + vysledek + " %.");
				
				
			} else {
				if ((i % prehravaniPoXGeneraci == 0) && (prehravaniPohybu)) {
					// vykreslen nejlepho een do mky
			
					/** pehrn nejzdatnjho genotypu:*/
					/** pevod genotypu na objekt Pravidlo): */
					Pravidlo p = new Pravidlo(velikostOkoli);
					for (int m = 0; m < p.getTabulkaOkoli().size(); m++) {
						Konfigurace konf = new Konfigurace(velikostOkoli);
						konf.setPosloupnost(p.getTabulkaOkoli().get(m));
						p.zapisVysledek(populace.get(i).getKoren().provedSeBezZobrazeni(konf));
					}
					PocitacCA pocitac = new PocitacCA();
					List<Konfigurace> konfList;
					konfList = new ArrayList<Konfigurace>();
					konfList.add(new Konfigurace(pocetAutomatuVRade));
					konfList.get(0).vygenerujNahodnouKonfiguraci();
					for (int in = 0; in < pocetRad; in++) {
						konfList.add(new Konfigurace(pocetAutomatuVRade));
						konfList.set(in + 1, pocitac.vypocitejDalsiRadku(konfList.get(in), p));
					}
					
					mrizka.setKonfigurace(konfList);
					mrizka.repaint();
				
					textarea.setText(nejlepsiGenotyp.getKoren().vypis());
					nejlepsiGenotyp.getKoren().setNejvyssiHloubka(nejlepsiGenotyp.getKoren());

				}
			
				List<Genotyp> novaPopulace = new ArrayList<Genotyp>();
				
				// zachovn nejlepho jedince v populaci
				if (zachovavatNejzdatnejsihoJedince) {
					Genotyp nejlepsiGenotyp2 = new Genotyp(nejlepsiGenotyp);
					novaPopulace.add(nejlepsiGenotyp2);
				}
				
				// vytvoen 1 - 5 nhodnch mutac nejlepho jedince
				if (zachovavatNejzdatnejsihoJedince) {
					int nahodnyPocet = nahodneKladneCislo(5) + 1;
					for (int m = 0; m < nahodnyPocet; m++) {
						Genotyp vybranyGenotyp = new Genotyp(nejlepsiGenotyp);
						// nhodn mutace na terminlech:
						VybranyGen vybranyGen = nahodnyTerminal(vybranyGenotyp);
						Gen gen = vybranyGen.getNadgen().geny.get(vybranyGen.getIndexPodgenu());
						
						// gen se zmn pln - vetn jeho podgen.
						if ((gen.hloubka - 1) == maximalniHloubkaStromuPoKrizeni) {	// terminl lze nahradit jen terminlem, pokud doshneme maximln hloubky
							gen = mnozinaTerminalu.get(nahodneKladneCislo(mnozinaTerminalu.size()));
						} else {
							Gen f = mnozinaFunkci.get(nahodneKladneCislo(mnozinaFunkci.size()));
							gen = new Funkce(f.prikaz, f.arita, gen.hloubka, maximalniHloubkaStromuPoKrizeni, mnozinaTerminalu, mnozinaFunkci, metodaInicializace);
						}
						
						vybranyGen.getNadgen().geny.set(vybranyGen.getIndexPodgenu(), gen);
						vybranyGenotyp.getKoren().opravHloubku();
						novaPopulace.add(vybranyGenotyp);
					}
				}
	
				// reprodukce
				for (int j = 0; j < pocetReprodukci; j++) {
					
					Genotyp vybranyGenotyp = vyberPodlePoradi();
					novaPopulace.add(vybranyGenotyp);
				}
				
				// ken
				for (int j = 0; j < pocetKrizeni; j++) {
					
					Genotyp vybranyGenotyp1 = vyberPodlePoradi();
					Genotyp vybranyGenotyp2 = vyberPodlePoradi();
					
					VybranyGen vybranyGen1 = nahodnyGen(vybranyGenotyp1);
					VybranyGen vybranyGen2 = nahodnyGen(vybranyGenotyp2);
					
					vybranyGen1.getNadgen().setHloubka(0);
					vybranyGen2.getNadgen().setHloubka(0);
					
					vybranyGen1.getNadgen().opravHloubku();
					vybranyGen2.getNadgen().opravHloubku();
					
					vybranyGen1.getNadgen().setNejvyssiHloubka(vybranyGen1.getNadgen());
					int maximalniHloubkaNadgenu1 = Gen.nejvyssiHloubka;
					
					vybranyGen2.getNadgen().setNejvyssiHloubka(vybranyGen2.getNadgen());
					int maximalniHloubkaNadgenu2 = Gen.nejvyssiHloubka;
					
					if ((maximalniHloubkaNadgenu1 + maximalniHloubkaNadgenu2) <= maximalniHloubkaStromuPoKrizeni) {
						
						Gen gen1 = vybranyGen1.getNadgen().geny.get(vybranyGen1.getIndexPodgenu());
						Gen gen2 = vybranyGen2.getNadgen().geny.get(vybranyGen2.getIndexPodgenu());
						
						vybranyGen1.getNadgen().geny.set(vybranyGen1.getIndexPodgenu(), gen2);
						vybranyGen2.getNadgen().geny.set(vybranyGen2.getIndexPodgenu(), gen1);
						
						vybranyGenotyp1.getKoren().opravHloubku();
						vybranyGenotyp2.getKoren().opravHloubku();
						
						novaPopulace.add(vybranyGenotyp1);
						novaPopulace.add(vybranyGenotyp2);
						
						
					} else {
						j--;
					}
				}
				
				// mutace:
				for (int j = 0; j < pocetMutaci; j++) {
					
					Genotyp vybranyGenotyp = vyberPodlePoradi();
					
					VybranyGen vybranyGen = nahodnyGen(vybranyGenotyp);
					
					Gen gen = vybranyGen.getNadgen().geny.get(vybranyGen.getIndexPodgenu());
					
					// gen se zmn pln - vetn jeho podgen.
					if ((gen.hloubka - 1) == maximalniHloubkaStromuPoKrizeni) {	// terminl lze nahradit jen terminlem, pokud doshneme maximln hloubky
						gen = mnozinaTerminalu.get(nahodneKladneCislo(mnozinaTerminalu.size()));
					} else {
						Gen f = mnozinaFunkci.get(nahodneKladneCislo(mnozinaFunkci.size()));
						gen = new Funkce(f.prikaz, f.arita, gen.hloubka, maximalniHloubkaStromuPoKrizeni, mnozinaTerminalu, mnozinaFunkci, metodaInicializace);
					}
					
					vybranyGen.getNadgen().geny.set(vybranyGen.getIndexPodgenu(), gen);
					vybranyGenotyp.getKoren().opravHloubku();
					
					novaPopulace.add(vybranyGenotyp);
				}
				
				populace = novaPopulace;			// nahrazen pvodn populace novou populac (generac)
			}
		}
	}
	
	/**
	 * Metoda vracejc z genotypu nhodn gen jako nadgen a pro nj index vybranho genu. Jako nadgen me vrtit i koen.
	 */
	private VybranyGen nahodnyGen(Genotyp genotyp) {
		
		Gen gen = genotyp.getKoren();		// vbr koene
		gen.setNejvyssiHloubka(gen);
		double p = 1.0/Gen.nejvyssiHloubka;
		
		// ppad, kdy je nejvy hloubka == 1, znamen, e musme vybrat terminl
		if (Gen.nejvyssiHloubka == 1) {
			VybranyGen vg = new VybranyGen();
			vg.setNadgen(gen);
			vg.setIndexPodgenu(nahodneKladneCislo(gen.geny.size()));
			return vg;
		}
		
		boolean funkce = false;
		if (nahodneKladneCisloDouble(1) < pravdepodobnostKrizeniVUzluFunkce) funkce = true;
		
		while (p <= 1) {
			if (nahodneKladneCisloDouble(1) < p) {							// pokud zvaujeme tento uzel
				if (! funkce) {												// m-li bt vybrn terminl
					List<Integer> indexy = new ArrayList<Integer>(0);
					for (int i = 0; i < gen.geny.size(); i++) {
						if (! gen.geny.get(i).isJeFunkce()) {
							indexy.add(i);
						}
					}
					if (indexy.size() != 0) {
						int i = indexy.get(nahodneKladneCislo(indexy.size()));
						VybranyGen vg = new VybranyGen();
						vg.setNadgen(gen);
						vg.setIndexPodgenu(i);
						return vg;
					}
					p = p + p;
					
					
					gen = gen.geny.get(nahodneKladneCislo(gen.geny.size()));
				} else {														// m-li bt vybrna funkce
					List<Integer> indexy = new ArrayList<Integer>(0);
					for (int i = 0; i < gen.geny.size(); i++) {
						if (gen.geny.get(i).isJeFunkce()) {
							indexy.add(i);
						}
					}
					if (indexy.size() != 0) {
						int i = indexy.get(nahodneKladneCislo(indexy.size()));
						VybranyGen vg = new VybranyGen();
						vg.setNadgen(gen);
						vg.setIndexPodgenu(i);
						return vg;
					}
					
					gen = genotyp.getKoren();
					break;
				}
			}
		}
		
		VybranyGen vg = new VybranyGen();
		vg.setNadgen(gen);
		vg.setIndexPodgenu(nahodneKladneCislo(gen.geny.size()));
		return vg;

	}
	
	/**
	 * Metoda vracejc z genotypu nhodn TERMINL jako VybranyGen s nadgenem vybranho genu a pro nj index vybranho genu.
	 */
	private VybranyGen nahodnyTerminal(Genotyp genotyp) {
		
		Gen gen = genotyp.getKoren();		// vbr koene
		gen.setNejvyssiHloubka(gen);
		double p = 1.0/Gen.nejvyssiHloubka;
		
		// ppad, kdy je nejvy hloubka == 1, znamen, e u vybrme terminl
		if (Gen.nejvyssiHloubka == 1) {
			VybranyGen vg = new VybranyGen();
			vg.setNadgen(gen);
			vg.setIndexPodgenu(nahodneKladneCislo(gen.geny.size()));
			return vg;
		}
		
		while (p <= 1) {
			if (nahodneKladneCisloDouble(1) < p) {							// pokud zvaujeme tento uzel
				List<Integer> indexy = new ArrayList<Integer>(0);
				for (int i = 0; i < gen.geny.size(); i++) {
					if (! gen.geny.get(i).isJeFunkce()) {
						indexy.add(i);
					}
				}
				if (indexy.size() != 0) {
					int i = indexy.get(nahodneKladneCislo(indexy.size()));
					VybranyGen vg = new VybranyGen();
					vg.setNadgen(gen);
					vg.setIndexPodgenu(i);
					return vg;
				}
				p = p + p;
				gen = gen.geny.get(nahodneKladneCislo(gen.geny.size()));
			}
		}
		
		VybranyGen vg = new VybranyGen();
		vg.setNadgen(gen);
		vg.setIndexPodgenu(nahodneKladneCislo(gen.geny.size()));
		return vg;

	}
	
	/**
	 * Metoda turnajov selekce pro 2 jedince
	 */
	/*private Genotyp turnajovaSelekce() {
		int index1 = nahodneKladneCislo(populace.size());
		int index2 = nahodneKladneCislo(populace.size());
		int index3 = nahodneKladneCislo(populace.size());
		while (index1 == index2 || index1 == index3 || index2 == index3) {
			index2 = nahodneKladneCislo(populace.size());
			index3 = nahodneKladneCislo(populace.size());
		}
		if (populace.get(index1).getZdatnost() >= populace.get(index2).getZdatnost()) {
			return new Genotyp(populace.get(index1));
		} else {
			return new Genotyp(populace.get(index2));
		}
	}*/
	
	/**
	 * Metoda turnajov selekce pro 3 jedince
	 */
	/*private Genotyp turnajovaSelekce3() {
		int pocet = 3;
		int[] poleIndexu = new int[pocet];
		
		for (int i = 0; i < pocet; i++) {
			poleIndexu[i] = nahodneKladneCislo(populace.size());
			
			for (int j = 0; j < i; j++) {
				if (poleIndexu[j] == poleIndexu[i]) {
					poleIndexu[i] = nahodneKladneCislo(populace.size());
					j = 0;
				}
			}
		}
		
		Genotyp vitez = populace.get(poleIndexu[0]);
		
		for (int i = 0; i < pocet; i++) {
			if (vitez.getZdatnost() < populace.get(poleIndexu[i]).getZdatnost()) {
				vitez = populace.get(poleIndexu[i]);
			}
		}
		
		return new Genotyp(vitez);
	}*/
	
	/**
	 * Metoda selekce podle poad (modifikovan vzhledem ke struktue zdatnost v populaci):
	 * 
	 * - kad jedinec se zdatnost 0 bude do populace pro vbr zahrnut prv tikrt,
	 * - jedinci se zdatnost vy ne 0 budou podle tto zdatnosti seazeni od n do 1
	 *  (kde n je poet jedinc se zdatnost vy ne 0) sestupn podle zdatnosti a budou
	 *  do populace pro vbr pidni tolikrt, jak mli poad.
	 *  Pokud neexistuj dn jedinci s nulovou zdatnost, nebudou se tam pidvat.
	 */
	private Genotyp vyberPodlePoradi() {

		// 1) vytvoen nov populace pro vbr (probhne jen pi prvnm vbru v rmci jedn generace):
		if (novaGenerace) {
			// zaloen nov populace pro vbr:
			populaceProVyber = new ArrayList<Genotyp>(0);
			List<Genotyp> populaceProPoradi = new ArrayList<Genotyp>(0);
			int pocetNulovych = 0;
			
			// zjitn zdatnosti:
			for (int i = 0; i < populace.size(); i++) {
				if (populace.get(i).getZdatnost() == 0) {
					pocetNulovych++;
				} else {
					populaceProPoradi.add(new Genotyp(populace.get(i)));
				}
			}
			
			// seazen jedinc podle zdatnosti:
			quicksort(populaceProPoradi, 0, populaceProPoradi.size());
			// vytvoen 1 - N kopi kadho jedince dle jeho poad a vloen do nov populace:
			for (int i = 0; i < populaceProPoradi.size(); i++) {
				for (int j = populaceProPoradi.size(); j > 0; j--) {
					populaceProVyber.add(new Genotyp(populaceProPoradi.get(i)));
				}
			}
			
			// k vloen nulovch jedinc do populace pro vbr:
			if (pocetNulovych != 0) {
				int k = (int) Math.ceil((populaceProVyber.size()/pocetNulovych)/10);
				for (int i = 0; i < populace.size(); i++) {
					if (populace.get(i).getZdatnost() == 0) {
						for (int j = 0; j < k; j++) {		
							populaceProVyber.add(new Genotyp(populace.get(i)));
						}
					}
				}
			}
			novaGenerace = false;
		}
		
		// 2) vbr podle poad z populace pro vbr:
		return new Genotyp(populaceProVyber.get(nahodneKladneCislo(populaceProVyber.size())));
	}
	
	/**
	 * Metoda pro vytvoen poten populace v zvislosti na zadan metod inicializace.
	 */
	private void vytvorPocatecniPopulaci(List<Gen> mnozinaTerminalu, List<Gen> mnozinaFunkci, int maximalniHloubka, int velikostPopulace, int metodaInicializace) {

		for (int i = 0; i < velikostPopulace; i++) {
			Genotyp genotyp = new Genotyp(maximalniHloubka, mnozinaTerminalu, mnozinaFunkci, metodaInicializace);
			populace.add(genotyp);
		}
		
	}
	
	/**
	 * Metoda pro provdn zadanho een. Postupn zpracuje textov zadan een na funkn genotyp (strom gen) a nsledn jej provd.
	 */
	public void provedeniGenotypu(Mrizka mrizka, int pocetRad, int maximalniHloubka, int velikostOkoli, int pocetAutomatuVRade, JTextArea textAreaReseni) throws BadLocationException {
		
		Funkce koren = sestavStrom(textAreaReseni.getText(), maximalniHloubka);

		if (koren != null) {
			
			try {

			// pevod genotypu na objekt Pravidlo:
			Pravidlo p = new Pravidlo(velikostOkoli);
			for (int m = 0; m < p.getTabulkaOkoli().size(); m++) {
				Konfigurace konf = new Konfigurace(velikostOkoli);
				konf.setPosloupnost(p.getTabulkaOkoli().get(m));
				p.zapisVysledek(koren.provedSeBezZobrazeni(konf));
			}
			
			// vpoet a vykreslen CA:
			PocitacCA pocitac = new PocitacCA();
			List<Konfigurace> konfList;
			konfList = new ArrayList<Konfigurace>();
			konfList.add(new Konfigurace(pocetAutomatuVRade));
			boolean[] posloupnost = new boolean[pocetAutomatuVRade];
			for (int it = 0; it < mrizka.getPocetSloupcu(); it++) {
				posloupnost[it] = mrizka.getProstor()[it][0].x;
			}
			konfList.get(0).setPosloupnost(posloupnost);
			
			for (int in = 0; in < pocetRad; in++) {
				konfList.add(new Konfigurace(pocetAutomatuVRade));
				konfList.set(in + 1, pocitac.vypocitejDalsiRadku(konfList.get(in), p));
			}
			
			mrizka.setKonfigurace(konfList);
			mrizka.repaint();
			
			} catch (IndexOutOfBoundsException io) {
				System.out.println("Zadan een obsahuje nepovolen znaky nebo pkazy.");
			}
		} else {
			System.out.println("Zadan een obsahuje nepovolen znaky nebo pkazy.");
		}

	}
	
	/**
	 * Metoda pro upesnn vsledku. Pevezme etzec z pole Vsledn een, pevede jej na genotyp a nsledn na pravidlo
	 * a otestuje na zadanm potu potench konfigurac. Dle nastaven (ne)vykresluje een uivateli.
	 * Vsledkem je procentuln vyjden spnost algoritmu.
	 */
	public void testGenotypu(Mrizka mrizka, int pocetRad, int pocetKonfiguraci, int maximalniHloubka, int velikostOkoli, int pocetAutomatuVRade, JTextArea textAreaReseni) throws BadLocationException {
		
		Funkce koren = sestavStrom(textAreaReseni.getText(), maximalniHloubka);

		if (koren != null) {
			
			System.out.println("Test zdatnosti byl zahjen. Pokejte, prosm...");
			stop = false;
			
			try {

			// pevod genotypu na objekt Pravidlo:
			Pravidlo p = new Pravidlo(velikostOkoli);
			for (int m = 0; m < p.getTabulkaOkoli().size(); m++) {
				Konfigurace konf = new Konfigurace(velikostOkoli);
				konf.setPosloupnost(p.getTabulkaOkoli().get(m));
				p.zapisVysledek(koren.provedSeBezZobrazeni(konf));
			}
			
			// otestovn na zadanm potu konfigurac:
			int zdatnost = 0;
			PocitacCA pocitac = new PocitacCA();
			List<Konfigurace> konfList;
			
			for (int ij = 0; ij < pocetKonfiguraci; ij++) {
				
				if (stop) {
					break;
				}
				
				konfList = new ArrayList<Konfigurace>();
				konfList.add(new Konfigurace(pocetAutomatuVRade));
				konfList.get(0).vygenerujNahodnouKonfiguraci();
				boolean prevladajiciBarva = pocitac.zjistiPrevladajiciBarvu(konfList.get(0));
				boolean ukonciPredcasne = false;
				for (int in = 0; in < pocetRad; in++) {
					if (konfList.get(in).getPorovnani() == 20 || konfList.get(in).getPorovnani() == 21) {	// barva stejn jako u pedchoz dky - dosaeno konce
						ukonciPredcasne = true;
						if (prevladajiciBarva == true && konfList.get(in).getPorovnani() == 21) {
							// bl vyla a bl je pevldajc
							zdatnost = zdatnost + 1;
						} else if (prevladajiciBarva == false && konfList.get(in).getPorovnani() == 20) {
							// ern vyla a ern je pevldajc
							zdatnost = zdatnost + 1;
						}
						break;
					} else if (konfList.get(in).getPorovnani() == 3) { // barva rozdln oproti pedchoz dce -> proukovn
						ukonciPredcasne = true;
						break;
					}
					konfList.add(new Konfigurace(pocetAutomatuVRade));
					konfList.set(in + 1, pocitac.vypocitejDalsiRadku(konfList.get(in), p));
				}
				
				if (! ukonciPredcasne) {
					if (pocitac.zjistiBarvuRadky(konfList.get(konfList.size() - 1)) == 0 && prevladajiciBarva == false) {
						zdatnost = zdatnost + 1;
					} else if (pocitac.zjistiBarvuRadky(konfList.get(konfList.size() - 1)) == 1 && prevladajiciBarva == true) {
						zdatnost = zdatnost + 1;
					}
				}
			}
			
			// vpis vsledku:
			System.out.println("Dosaen zdatnost: " + zdatnost + ".");
			double vysledek = ((double)zdatnost / (double)pocetKonfiguraci) * 100;
			System.out.println("Vpoet byl ukonen.");
			System.out.println("VSLEDN SP̊NOST JE " + vysledek + " %.");
			
			} catch (IndexOutOfBoundsException io) {
				System.out.println("Zadan een obsahuje nepovolen znaky nebo pkazy.");
			}
		} else {
			System.out.println("Zadan een obsahuje nepovolen znaky nebo pkazy.");
		}

	}
	
	/**
	 * Funkce pro sestaven stromu (vrac jeho koen, obsahujc vechny poduzly) z textovho etzce (se zvorkami, rkami apod.).
	 */
	public Funkce sestavStrom (String text, int maximalniHloubka) {
		List<Gen> mnozinaTerminalu = new ArrayList<Gen>();
		List<Gen> mnozinaFunkci = new ArrayList<Gen>();
		  
		Gen s = new Terminal("S");
		Gen z1 = new Terminal("Z1");
		Gen z2 = new Terminal("Z2");
		Gen z3 = new Terminal("Z3");
		Gen v1 = new Terminal("V1");
		Gen v2 = new Terminal("V2");
		Gen v3 = new Terminal("V3");
		 
		mnozinaTerminalu.add(s);
		mnozinaTerminalu.add(z1);
		mnozinaTerminalu.add(z2);
		mnozinaTerminalu.add(z3);
		mnozinaTerminalu.add(v1);
		mnozinaTerminalu.add(v2);
		mnozinaTerminalu.add(v3);
		  
		Gen and = new Gen("AND", 2);
		Gen nand = new Gen("NAND", 2);
		Gen or = new Gen("OR", 2);
		Gen nor = new Gen("NOR", 2);
		Gen xor = new Gen("XOR", 2);
		Gen xnor = new Gen("XNOR", 2);
		Gen not = new Gen("NOT", 1);
		Gen ifg = new Gen("IF", 3);
		  
		mnozinaFunkci.add(and);
		mnozinaFunkci.add(nand);
		mnozinaFunkci.add(or);
		mnozinaFunkci.add(nor);
		mnozinaFunkci.add(xor);
		mnozinaFunkci.add(xnor);
		mnozinaFunkci.add(not);
		mnozinaFunkci.add(ifg);
		
		String retezec = "";
		
		// pi zadn przdnho etzce
		if(text.length() == 0) {
			System.out.println("Nebyl zadn dn program pro pehrn.");
			return null;
		}
		
		// odstrann zvorek, mezer a rek
		for (int i = 0; i < text.length(); i++) {
			if (! (text.charAt(i) == ' ' | text.charAt(i) == ',' | text.charAt(i) == '(' | text.charAt(i) == ')' )) {
				retezec = retezec + text.charAt(i);
			}
		}
		
		String prikaz = "";
		List<Funkce> seznamFunkci = new ArrayList<Funkce>();	// zsobnk
		Funkce koren = null;
		Terminal terminal;
		
		// rozpozn zadan pkaz
		for (int i = 0; i < retezec.length(); i++) {
			prikaz = prikaz + retezec.charAt(i);
			
			switch (prikaz) {
					
				case "AND" :
					prikaz = "";
					
					if (seznamFunkci.size() != 0) {
						
						while (seznamFunkci.get(seznamFunkci.size() - 1).arita == seznamFunkci.get(seznamFunkci.size() - 1).geny.size()) {
							seznamFunkci.remove(seznamFunkci.size() - 1);
						}
						
						Funkce funkce = new Funkce("AND", 2, seznamFunkci.size(), maximalniHloubka, mnozinaTerminalu, mnozinaFunkci);
						seznamFunkci.get(seznamFunkci.size() - 1).geny.add(funkce);
						seznamFunkci.add(funkce);
						
					} else {
						seznamFunkci.add(new Funkce("AND", 2, 0, maximalniHloubka, mnozinaTerminalu, mnozinaFunkci));
						koren = seznamFunkci.get(0);	
					}

					break;
					
				case "NAND" :
					prikaz = "";
					
					if (seznamFunkci.size() != 0) {
						
						while (seznamFunkci.get(seznamFunkci.size() - 1).arita == seznamFunkci.get(seznamFunkci.size() - 1).geny.size()) {
							seznamFunkci.remove(seznamFunkci.size() - 1);
						}
						
						Funkce funkce = new Funkce("NAND", 2, seznamFunkci.size(), maximalniHloubka, mnozinaTerminalu, mnozinaFunkci);
						seznamFunkci.get(seznamFunkci.size() - 1).geny.add(funkce);
						seznamFunkci.add(funkce);
						
					} else {
						seznamFunkci.add(new Funkce("NAND", 2, 0, maximalniHloubka, mnozinaTerminalu, mnozinaFunkci));
						koren = seznamFunkci.get(0);	
					}

					break;
					
				case "OR" :
					prikaz = "";
					
					if (seznamFunkci.size() != 0) {
						
						while (seznamFunkci.get(seznamFunkci.size() - 1).arita == seznamFunkci.get(seznamFunkci.size() - 1).geny.size()) {
							seznamFunkci.remove(seznamFunkci.size() - 1);
						}
						
						Funkce funkce = new Funkce("OR", 2, seznamFunkci.size(), maximalniHloubka, mnozinaTerminalu, mnozinaFunkci);
						seznamFunkci.get(seznamFunkci.size() - 1).geny.add(funkce);
						seznamFunkci.add(funkce);
						
					} else {
						seznamFunkci.add(new Funkce("OR", 2, 0, maximalniHloubka, mnozinaTerminalu, mnozinaFunkci));
						koren = seznamFunkci.get(0);	
					}

					break;
					
				case "NOR" :
					prikaz = "";
					
					if (seznamFunkci.size() != 0) {
						
						while (seznamFunkci.get(seznamFunkci.size() - 1).arita == seznamFunkci.get(seznamFunkci.size() - 1).geny.size()) {
							seznamFunkci.remove(seznamFunkci.size() - 1);
						}
						
						Funkce funkce = new Funkce("NOR", 2, seznamFunkci.size(), maximalniHloubka, mnozinaTerminalu, mnozinaFunkci);
						seznamFunkci.get(seznamFunkci.size() - 1).geny.add(funkce);
						seznamFunkci.add(funkce);
						
					} else {
						seznamFunkci.add(new Funkce("NOR", 2, 0, maximalniHloubka, mnozinaTerminalu, mnozinaFunkci));
						koren = seznamFunkci.get(0);	
					}

					break;
					
				case "NOT" :
					prikaz = "";
					
					if (seznamFunkci.size() != 0) {
						
						while (seznamFunkci.get(seznamFunkci.size() - 1).arita == seznamFunkci.get(seznamFunkci.size() - 1).geny.size()) {
							seznamFunkci.remove(seznamFunkci.size() - 1);
						}
						
						Funkce funkce = new Funkce("NOT", 1, seznamFunkci.size(), maximalniHloubka, mnozinaTerminalu, mnozinaFunkci);
						seznamFunkci.get(seznamFunkci.size() - 1).geny.add(funkce);
						seznamFunkci.add(funkce);
						
					} else {
						seznamFunkci.add(new Funkce("NOT", 1, 0, maximalniHloubka, mnozinaTerminalu, mnozinaFunkci));
						koren = seznamFunkci.get(0);	
					}

					break;
					
				case "XOR" :
					prikaz = "";
					
					if (seznamFunkci.size() != 0) {
						
						while (seznamFunkci.get(seznamFunkci.size() - 1).arita == seznamFunkci.get(seznamFunkci.size() - 1).geny.size()) {
							seznamFunkci.remove(seznamFunkci.size() - 1);
						}
						
						Funkce funkce = new Funkce("XOR", 2, seznamFunkci.size(), maximalniHloubka, mnozinaTerminalu, mnozinaFunkci);
						seznamFunkci.get(seznamFunkci.size() - 1).geny.add(funkce);
						seznamFunkci.add(funkce);
						
					} else {
						seznamFunkci.add(new Funkce("XOR", 2, 0, maximalniHloubka, mnozinaTerminalu, mnozinaFunkci));
						koren = seznamFunkci.get(0);	
					}

					break;
					
				case "XNOR" :
					prikaz = "";
					
					if (seznamFunkci.size() != 0) {
						
						while (seznamFunkci.get(seznamFunkci.size() - 1).arita == seznamFunkci.get(seznamFunkci.size() - 1).geny.size()) {
							seznamFunkci.remove(seznamFunkci.size() - 1);
						}
						
						Funkce funkce = new Funkce("XNOR", 2, seznamFunkci.size(), maximalniHloubka, mnozinaTerminalu, mnozinaFunkci);
						seznamFunkci.get(seznamFunkci.size() - 1).geny.add(funkce);
						seznamFunkci.add(funkce);
						
					} else {
						seznamFunkci.add(new Funkce("XNOR", 2, 0, maximalniHloubka, mnozinaTerminalu, mnozinaFunkci));
						koren = seznamFunkci.get(0);	
					}

					break;
					
				case "IF" :
					prikaz = "";
					
					if (seznamFunkci.size() != 0) {
						
						while (seznamFunkci.get(seznamFunkci.size() - 1).arita == seznamFunkci.get(seznamFunkci.size() - 1).geny.size()) {
							seznamFunkci.remove(seznamFunkci.size() - 1);
						}
						
						Funkce funkce = new Funkce("IF", 3, seznamFunkci.size(), maximalniHloubka, mnozinaTerminalu, mnozinaFunkci);
						seznamFunkci.get(seznamFunkci.size() - 1).geny.add(funkce);
						seznamFunkci.add(funkce);
						
					} else {
						seznamFunkci.add(new Funkce("IF", 3, 0, maximalniHloubka, mnozinaTerminalu, mnozinaFunkci));
						koren = seznamFunkci.get(0);	
					}

					break;
					
					
				case "S" :
					while (seznamFunkci.get(seznamFunkci.size() - 1).arita == seznamFunkci.get(seznamFunkci.size() - 1).geny.size()) {
						seznamFunkci.remove(seznamFunkci.size() - 1);
					}
					prikaz = "";
					terminal = new Terminal("S");
					
					if (seznamFunkci.size() != 0) {
						seznamFunkci.get(seznamFunkci.size() - 1).geny.add(terminal);
					}

					break;
					
				case "Z1" :
					while (seznamFunkci.get(seznamFunkci.size() - 1).arita == seznamFunkci.get(seznamFunkci.size() - 1).geny.size()) {
						seznamFunkci.remove(seznamFunkci.size() - 1);
					}
					prikaz = "";
					terminal = new Terminal("Z1");
					
					if (seznamFunkci.size() != 0) {
						seznamFunkci.get(seznamFunkci.size() - 1).geny.add(terminal);
					}

					break;
					
				case "Z2" :
					while (seznamFunkci.get(seznamFunkci.size() - 1).arita == seznamFunkci.get(seznamFunkci.size() - 1).geny.size()) {
						seznamFunkci.remove(seznamFunkci.size() - 1);
					}
					prikaz = "";
					terminal = new Terminal("Z2");
					
					if (seznamFunkci.size() != 0) {
						seznamFunkci.get(seznamFunkci.size() - 1).geny.add(terminal);
					}

					break;
					
				case "Z3" :
					while (seznamFunkci.get(seznamFunkci.size() - 1).arita == seznamFunkci.get(seznamFunkci.size() - 1).geny.size()) {
						seznamFunkci.remove(seznamFunkci.size() - 1);
					}
					prikaz = "";
					terminal = new Terminal("Z3");
					
					if (seznamFunkci.size() != 0) {
						seznamFunkci.get(seznamFunkci.size() - 1).geny.add(terminal);
					}

					break;
					
				case "V1" :
					while (seznamFunkci.get(seznamFunkci.size() - 1).arita == seznamFunkci.get(seznamFunkci.size() - 1).geny.size()) {
						seznamFunkci.remove(seznamFunkci.size() - 1);
					}
					prikaz = "";
					terminal = new Terminal("V1");
					
					if (seznamFunkci.size() != 0) {
						seznamFunkci.get(seznamFunkci.size() - 1).geny.add(terminal);
					}

					break;
					
				case "V2" :
					while (seznamFunkci.get(seznamFunkci.size() - 1).arita == seznamFunkci.get(seznamFunkci.size() - 1).geny.size()) {
						seznamFunkci.remove(seznamFunkci.size() - 1);
					}
					prikaz = "";
					terminal = new Terminal("V2");
					
					if (seznamFunkci.size() != 0) {
						seznamFunkci.get(seznamFunkci.size() - 1).geny.add(terminal);
					}

					break;
					
				case "V3" :
					while (seznamFunkci.get(seznamFunkci.size() - 1).arita == seznamFunkci.get(seznamFunkci.size() - 1).geny.size()) {
						seznamFunkci.remove(seznamFunkci.size() - 1);
					}
					prikaz = "";
					terminal = new Terminal("V3");
					
					if (seznamFunkci.size() != 0) {
						seznamFunkci.get(seznamFunkci.size() - 1).geny.add(terminal);
					}

					break;
					
				default :
					break;
			}
		}
		
		return koren;
	}
	
	public int nahodneKladneCislo(int horniHranice) {		// vraci hodnoty 0 =< hodnota < horniHranice
		return (int)(Math.random() * horniHranice);
	}
	
	public double nahodneKladneCisloDouble(int horniHranice) {
		return Math.random() * horniHranice;
	}

	public JButton getButtonZastavit() {
		return buttonZastavit;
	}

	public void setButtonZastavit(JButton buttonZastavit) {
		this.buttonZastavit = buttonZastavit;
	}
	
	/**
	* quicksort - metoda pro seazen jedinc podle zdatnosti. Vyuvaj ji decimace a vbr podle poad.
	*/
	public static void quicksort (List<Genotyp> populace, int left, int right) {
		if (left < right) {
			int boundary = left;
			for(int i = left + 1; i < right; i++) {
				if (populace.get(i).getZdatnost() > populace.get(left).getZdatnost()) {
					swap(populace, i, ++boundary);
				}
			}
			swap(populace, left, boundary);
			quicksort(populace, left, boundary);
			quicksort(populace, boundary + 1, right);
		}    
	}

	private static void swap (List<Genotyp> populace, int left, int right) {
		Genotyp tmp = new Genotyp(populace.get(right));
		populace.set(right, populace.get(left));
		populace.set(left, tmp);
	}
	
}