package model;

import java.util.ArrayList;
import java.util.List;

/**
 * Tda reprezentujc pravidlo (tabulku), pro uloen informac, jak bude vsledek pi jakm okol (mon 3, 5 nebo 7 bunk)
 */
public class Pravidlo {
	
	// true = bl = 1, false = ern = 0.
	private List<boolean[]> tabulkaOkoli;
	private List<Boolean> tabulkaVysledku;
	private int velikostOkoli;
	
	public Pravidlo(int velikostOkoli) {
		this.velikostOkoli = velikostOkoli;
		this.tabulkaOkoli = new ArrayList<boolean[]>();
		this.tabulkaVysledku = new ArrayList<Boolean>();
		vygenerujTabulkuOkoli();
	}
	
	private void vygenerujTabulkuOkoli() {
		/*
		 * Okol = 3:
		 * 
		 * 0 0 0
		 * 0 0 1
		 * 0 1 0
		 * 0 1 1
		 * 1 0 0
		 * 1 0 1
		 * 1 1 0
		 * 1 1 1
		 * 
		 * Okol = 4:
		 * 
		 * 0 0 0 0
		 * 0 0 0 1
		 * 0 0 1 0
		 * 0 0 1 1
		 * 0 1 0 0
		 * 0 1 0 1
		 * 0 1 1 0
		 * 0 1 1 1
		 * 1 0 0 0
		 * 1 0 0 1
		 * 1 0 1 0
		 * 1 0 1 1
		 * 1 1 0 0
		 * 1 1 0 1
		 * 1 1 1 0
		 * 1 1 1 1
		 */
		
		// vytvoen pol:
		for (int i = 0; i < Math.pow(2, this.velikostOkoli); i++) {
			boolean[] pole = new boolean[this.velikostOkoli];
			this.tabulkaOkoli.add(pole);
		}
		
		// naplnn pol po sloupcch:
		for (int j = 0; j < this.velikostOkoli; j++) {
			boolean zapisovanaHodnota = false;
			double pocet = Math.pow(2, this.velikostOkoli) / (2 * Math.pow(2, j));
			int zapsanych = 0;
			for (int i = 0; i < Math.pow(2, this.velikostOkoli); i++) {
				this.tabulkaOkoli.get(i)[j] = zapisovanaHodnota;
				zapsanych++;
				if (zapsanych == pocet) {
					zapisovanaHodnota = ! zapisovanaHodnota;
					zapsanych = 0;
				}
			}
		}
	}
	
	public void zapisVysledek(Boolean vysledek) {
		tabulkaVysledku.add(vysledek);
	}

	public List<boolean[]> getTabulkaOkoli() {
		return tabulkaOkoli;
	}

	public void setTabulkaOkoli(List<boolean[]> tabulkaOkoli) {
		this.tabulkaOkoli = tabulkaOkoli;
	}

	public List<Boolean> getTabulkaVysledku() {
		return tabulkaVysledku;
	}

	public void setTabulkaVysledku(List<Boolean> tabulkaVysledku) {
		this.tabulkaVysledku = tabulkaVysledku;
	}

	public int getVelikostOkoli() {
		return velikostOkoli;
	}

	public void setVelikostOkoli(int velikostOkoli) {
		if (velikostOkoli % 2 == 0) {
			System.out.println("Neplatn velikost okol - okol mus mt velikost 3, 5, 7...");
		} else {
			this.velikostOkoli = velikostOkoli;
		}
	}
}
