package rozhrani;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.List;
import javax.swing.JPanel;
import model.Bunka;
import model.Konfigurace;

/**
 * Tda pedstavujc prostor celulrnch automat.
 */
public class Mrizka extends JPanel {

		private static final long serialVersionUID = 1L;
		
		private int pocetSloupcu = 149;
		private int pocetRadku = 150;
		private Color barvaSite = Color.GRAY;
		
		private int sirkaOkna = 747;
		private int vyskaOkna = 637;
		private int minimalniVelikost = 7;
		
		private int sirkaBunky = minimalniVelikost;
		private int vyskaBunky = minimalniVelikost;
		
		private Bunka[][] prostor;
		private List<Konfigurace> konfigurace;
		
		public Mrizka() {
			prostor = new Bunka[pocetSloupcu][pocetRadku];
		}
		
		public void nastavBunku(Point p, Bunka u) {
			prostor[p.x][p.y] = u;
		}
		
		public void nastavBunkuAVykresliBunku(Point p, Bunka u) {
			prostor[p.x][p.y] = u;
			kresliBunku(p, (Graphics2D)getGraphics());
		}
		
		private void kresliBunku(Point p, Graphics2D gr) {
			if (gr != null) {
				
				Bunka u = prostor[p.x][p.y];
				
				sirkaBunky = sirkaOkna / getPocetSloupcu();
				vyskaBunky = vyskaOkna / pocetRadku;
				
				if (sirkaBunky < minimalniVelikost) {
					sirkaBunky = minimalniVelikost;
				}
				if (vyskaBunky < minimalniVelikost) {
					vyskaBunky = minimalniVelikost;
				}
				
				Rectangle rect = new Rectangle(p.x * sirkaBunky, p.y * vyskaBunky, sirkaBunky, vyskaBunky);
				
				gr.setColor(barvaSite);
				gr.drawRect(rect.x, rect.y, rect.width, rect.height);
				
				rect.x++;
				rect.y++;
				rect.width -= 1;
				rect.height -=1;
				
				if (u != null) {
					u.kresli(gr, rect.x, rect.y, rect.width, rect.height);
				} else {
					gr.setColor(Color.WHITE);
					gr.fillRect(rect.x, rect.y, rect.width, rect.height);
				}
			}
		}
		
		public void paint(Graphics g) {		// metoda pro kreslen bunk a pro zmnu velikosti bunk pi zmn velikosti okna
			super.paint(g);
			
			if (konfigurace != null) {
				pocetSloupcu = konfigurace.get(0).getPosloupnost().length;
				pocetRadku = konfigurace.size();
				prostor = new Bunka[pocetSloupcu][pocetRadku];

				for (int i = 0; i < getPocetSloupcu(); i++) {
					for (int j = 0; j < pocetRadku; j++) {
						Point p = new Point();
						p.x = i;
						p.y = j;
						prostor[i][j] = new Bunka();
						prostor[i][j].x = konfigurace.get(j).getPosloupnost()[i];
						//nastavBunkuAVykresliBunku(p, prostor[i][j]);
						kresliBunku(p, (Graphics2D)g);
					}
				}
			} else {
				Point p = new Point();
				
				for (int i = 0; i < getPocetSloupcu(); i++) {
					for (int j = 0; j < pocetRadku; j++) {
						p.x = i;
						p.y = j;
						kresliBunku(p, (Graphics2D)g);
					}
				}
			}
		}
		
		public void reset() {
			prostor = new Bunka[pocetSloupcu][pocetRadku];
			smazPlatno();
		}
		
		public void smazPlatno() {
			Point p = new Point();
			
			for (int i = 0; i < getPocetSloupcu(); i++) {
				for (int j = 0; j < pocetRadku; j++) {
					p.x = i;
					p.y = j;
					nastavBunkuAVykresliBunku(p, null);
				}
			}
		}
		
		public Bunka[][] getProstor() {
			return prostor;
		}
		
		public int getPocetSloupcu() {
			return pocetSloupcu;
		}
		
		public int getPocetRadku() {
			return pocetRadku;
		}
		
		public int getVyskaBunky() {
			return vyskaBunky;
		}
		
		public int getSirkaBunky() {
			return sirkaBunky;
		}
		
		public void setVyskaBunky(int vyska) {
			vyskaBunky = vyska;
		}
		
		public void setSirkaBunky(int sirka) {
			sirkaBunky = sirka;
		}

		public Point getRozmerHraciPlochy() {
			return new Point(getPocetSloupcu(), pocetRadku);
		}

		public List<Konfigurace> getKonfigurace() {
			return konfigurace;
		}

		public void setKonfigurace(List<Konfigurace> konfigurace) {
			this.konfigurace = konfigurace;
		}
		
		public void setPocetRadku(int pocetRadku) {
			this.pocetRadku = pocetRadku;
			reset();
			repaint();
		}
		
		public void setPocetSloupcu(int pocetSloupcu) {
			this.pocetSloupcu = pocetSloupcu;
			reset();
			repaint();
		}
		
		public int getSirka() {
			return sirkaBunky*pocetSloupcu + 1;
		}
		
		public int getVyska() {
			return vyskaBunky*pocetRadku + 1;
		}
}