package rozhrani;

import genetika.Funkce;
import genetika.GenetickyAlgoritmus;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import model.Bunka;
import model.Konfigurace;
import model.PocitacCA;
import model.Pravidlo;

/**
 * Tda okna pro nastaven vlastnho pravidla.
 */
public class OknoPravidlo extends JFrame {

	private static final long serialVersionUID = 1L;
	private OknoAplikace oknoAplikace;
	private JPanel panelSousedstvi;
	private JPanel panelTlacitek;
	private JButton buttonUlozit;
	private JButton buttonNacist;
	private JButton buttonReset;
	private JButton buttonZrusit;
	private JButton buttonAplikovat;
	private int velikostBunky = 7;
	private List<Mrizka> mrizky;
	private GenetickyAlgoritmus genetickyAlgoritmus;
	Pravidlo pravidlo;
	List<boolean[]> tabulkaOkoli;
	
	
    /**
     * Metoda inicializujc vechny komponenty a listenery okna zobrazujcho uivatelsky definovan pravidla.
     */
	public OknoPravidlo(OknoAplikace oknoAplikace) {
		this.oknoAplikace = oknoAplikace;
		this.setLayout(new GridBagLayout());
		this.mrizky = new ArrayList<Mrizka>();
		this.genetickyAlgoritmus = new GenetickyAlgoritmus();
		
		vytvorOknoPodleSousedstvi();

        buttonUlozit = new javax.swing.JButton();
        buttonNacist = new javax.swing.JButton();
        buttonReset = new javax.swing.JButton();
        buttonZrusit = new javax.swing.JButton();
        buttonAplikovat = new javax.swing.JButton();
        
        buttonUlozit.setText("Uloit");
        buttonNacist.setText("Nast");
        buttonReset.setText("Reset");
        buttonZrusit.setText("Zruit");
        buttonAplikovat.setText("Aplikovat");
        
        panelTlacitek.add(buttonUlozit);
        panelTlacitek.add(buttonNacist);
        panelTlacitek.add(buttonReset);
        panelTlacitek.add(buttonZrusit);
        panelTlacitek.add(buttonAplikovat);
        
        pridejListenery();
        
        setTitle("Vlastn pravidlo");
        pack();
		setVisible(true);
	}
	
	/**
	 * Vytvo okno podle velikosti sousedstv a zrove,
	 * pokud je v poli Vsledn een zadn textov etzec pravidla,
	 * pevede jej na tabulku a podle n vypln vsledn pole (jinak je ponech bl).
	 */
	public void vytvorOknoPodleSousedstvi() {
		panelTlacitek = new JPanel();
		panelSousedstvi = new JPanel();
		
		GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.BOTH;
        c.gridx = 0;
        c.gridy = 0;
        c.insets = new Insets(0, 0, 0, 0);
        c.anchor = GridBagConstraints.FIRST_LINE_START;
        this.add(panelSousedstvi, c);
        c.gridy = 1;
        this.add(panelTlacitek, c);
        
        panelTlacitek.setLayout(new FlowLayout());
        
        // jednotliv sousedstv a klikateln polka (mka 11)
        panelSousedstvi.setLayout(new GridBagLayout());
        
        // nov pravidlo:
        pravidlo = new Pravidlo(oknoAplikace.getVelikostOkoli());
        tabulkaOkoli = pravidlo.getTabulkaOkoli();

        for (int i = 0; i < tabulkaOkoli.size(); i++) {
            		
        	Mrizka mrizka = new Mrizka();
        	mrizka.setSirkaBunky(velikostBunky);
        	mrizka.setVyskaBunky(velikostBunky);
    		mrizka.setPreferredSize(new Dimension(velikostBunky * oknoAplikace.getVelikostOkoli(), velikostBunky));
    		JPanel panelMrizka = new JPanel();
    		panelMrizka.setVisible(true);
    		JScrollPane scrollPane = new javax.swing.JScrollPane(mrizka);
            scrollPane.setHorizontalScrollBarPolicy(javax.swing.JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
            scrollPane.setVerticalScrollBarPolicy(javax.swing.JScrollPane.VERTICAL_SCROLLBAR_NEVER);
            scrollPane.setBounds(0, 0, 750, 640);
            panelMrizka.add(scrollPane);
            
            for (int j = 0; j < tabulkaOkoli.get(i).length; j++) {
	            Bunka bunka = new Bunka();
				bunka.setPozice(new Point(j, 0));
				if (tabulkaOkoli.get(i)[j]) {
					bunka.setBarva(Color.WHITE);
				} else {
					bunka.setBarva(Color.BLACK);
				}
				mrizka.nastavBunkuAVykresliBunku(new Point(j, 0), bunka);
            }
            
            if (oknoAplikace.getVelikostOkoli() == 6) {
               	if (i >= 32) {		// rozdlen do 2 sloupc v ppad, e by se pravidla nevela na obrazovku
                   	c.gridx = 2;
                   	c.gridy = i % 32;
               	} else {
                   	c.gridx = 0;
                   	c.gridy = i;
               	}
           	} else if (oknoAplikace.getVelikostOkoli() == 7) {
               	if (i >= 96) {		// rozdlen do 4 sloupc v ppad, e by se pravidla nevela na obrazovku
                   	c.gridx = 6;
                   	c.gridy = i % 96;
               	} else if (i >= 64) {
                   	c.gridx = 4;
                   	c.gridy = i % 64;
               	} else if (i >= 32) {
                   	c.gridx = 2;
                   	c.gridy = i % 32;
               	} else {
                   	c.gridx = 0;
                   	c.gridy = i;
               	}
           	} else {
               	c.gridx = 0;
               	c.gridy = i;
           	}
            
        	panelSousedstvi.add(panelMrizka, c);
        	
        	
        	// klikac polko:
        	
        	final Mrizka m = new Mrizka();
        	mrizky.add(m);
        	m.setSirkaBunky(velikostBunky);
        	m.setVyskaBunky(velikostBunky);
        	m.nastavBunku(new Point(0, 0), new Bunka());
    		m.setPreferredSize(new Dimension(velikostBunky, velikostBunky));
    		JPanel panelM = new JPanel();
    		panelM.setVisible(true);
    		JScrollPane scrollPaneM = new javax.swing.JScrollPane(m);
    		scrollPaneM.setHorizontalScrollBarPolicy(javax.swing.JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    		scrollPaneM.setVerticalScrollBarPolicy(javax.swing.JScrollPane.VERTICAL_SCROLLBAR_NEVER);
    		//scrollPaneM.setBounds(0, 0, 750, 640);
    		panelM.add(scrollPaneM);
    		
            if (oknoAplikace.getVelikostOkoli() == 6) {
               	if (i >= 32) {		// rozdlen do 2 sloupc v ppad, e by se pravidla nevela na obrazovku
                   	c.gridx = 3;
                   	c.gridy = i % 32;
               	} else {
                   	c.gridx = 1;
                   	c.gridy = i;
               	}
           	} else if (oknoAplikace.getVelikostOkoli() == 7) {
               	if (i >= 96) {		// rozdlen do 4 sloupc v ppad, e by se pravidla nevela na obrazovku
                   	c.gridx = 7;
                   	c.gridy = i % 96;
               	} else if (i >= 64) {
                   	c.gridx = 5;
                   	c.gridy = i % 64;
               	} else if (i >= 32) {
                   	c.gridx = 3;
                   	c.gridy = i % 32;
               	} else {
                   	c.gridx = 1;
                   	c.gridy = i;
               	}
           	} else {
               	c.gridx = 1;
               	c.gridy = i;
           	}
            
            panelSousedstvi.add(panelM, c);
            
            m.addMouseListener(new MouseListener() {
    			
    			@Override
    			public void mouseReleased(MouseEvent arg0) {
    			}
    			
    			@Override
    			/**
    			 * Metoda pro zadvn celulrnch automat uivatelem.
    			 */
    			public void mousePressed(MouseEvent e) {
    					Bunka bunka = m.getProstor()[0][0];
    					if (bunka.x == true) {
    						bunka.setBarva(Color.BLACK);
    					} else {
    						bunka.setBarva(Color.WHITE);
    					}

    					m.nastavBunkuAVykresliBunku(new Point(0, 0), bunka);
    			}

				@Override
				public void mouseClicked(MouseEvent arg0) {
					
				}

				@Override
				public void mouseEntered(MouseEvent e) {
					
				}

				@Override
				public void mouseExited(MouseEvent e) {
					
				}
            }); 
        }
        
        // pokud bylo vyplnno textov pole v Okn Aplikace pro etzec GP, bude v mce zobrazeno jeho vsledn pravidlo:
    	Funkce koren = null;
    	if (! oknoAplikace.getReseni().equals("")) {
    		try {
        		koren = genetickyAlgoritmus.sestavStrom(oknoAplikace.getReseni(), oknoAplikace.getMaximalniHloubka());
            	// pevod genotypu na objekt Pravidlo:
        		pravidlo = new Pravidlo(oknoAplikace.getVelikostOkoli());
            	for (int o = 0; o < pravidlo.getTabulkaOkoli().size(); o++) {
            		Konfigurace konf = new Konfigurace(oknoAplikace.getVelikostOkoli());
            		konf.setPosloupnost(pravidlo.getTabulkaOkoli().get(o));
            		pravidlo.zapisVysledek(koren.provedSeBezZobrazeni(konf));
            	}

    		} catch (IndexOutOfBoundsException io) {
				System.out.println("Zadan een obsahuje nepovolen znaky nebo pkazy.");
			}
    		
        	// zapsn vsledk do jednotlivch polek (mek):
    		for (int j = 0; j < pravidlo.getTabulkaVysledku().size(); j++) {
    			Bunka b = new Bunka();
    			if (pravidlo.getTabulkaVysledku().get(j) == true) {
    				b.setBarva(Color.WHITE);
    			} else {
    				b.setBarva(Color.BLACK);
    			}
    			mrizky.get(j).nastavBunkuAVykresliBunku(new Point(0, 0), b);
    		}
    	}
	}
	
	public void pridejListenery() {
		
		buttonUlozit.addActionListener(new java.awt.event.ActionListener() {
        	/**
        	 * Metoda pro uloen pravidla do textovho souboru.
        	 */
            public void actionPerformed(java.awt.event.ActionEvent evt) {

            	String text = Integer.toString(oknoAplikace.getVelikostOkoli());
            	
            	for (int i = 0; i < tabulkaOkoli.size(); i++) {
            		for (int j = 0; j < tabulkaOkoli.get(0).length; j++) {
            			if (tabulkaOkoli.get(i)[j]) {
            				text = text + "1";
            			} else {
            				text = text + "0";
            			}
            		}
            		
            		// pidn vsledku za sekvenci okol:
            		if (mrizky.get(i).getProstor()[0][0].getBarva() == Color.WHITE) {
            			text = text + "1";
            		} else {
            			text = text + "0";
            		}
            	}

                JFileChooser chooser = new JFileChooser();
                chooser.setCurrentDirectory( new File( "./") );
                int actionDialog = chooser.showSaveDialog(chooser.getParent());
                if (actionDialog == JFileChooser.APPROVE_OPTION)
                {
                    File fileName = new File(chooser.getSelectedFile() + ".txt" );

                    if(fileName.exists())
                    {
                        actionDialog = JOptionPane.showConfirmDialog(chooser.getParent(), "Nahradit existujc soubor?");
                        if (actionDialog == JOptionPane.NO_OPTION)
                            return;
                    }
                    try
                    {
                        BufferedWriter out = new BufferedWriter(new FileWriter(fileName));

                            out.write(text);
                            out.close();
                    }
                    catch(Exception e)
                    {
                         System.err.println("Error: " + e.getMessage());
                    }
                }
            }
        });
		
        buttonNacist.addActionListener(new java.awt.event.ActionListener() {
        	/**
        	 * Metoda pro naten pravidla z textovho souboru.
        	 */
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JFileChooser chooser = new JFileChooser();
                chooser.setCurrentDirectory( new File( "./") );
                int actionDialog = chooser.showOpenDialog(chooser.getParent());
                if (actionDialog == JFileChooser.APPROVE_OPTION)
                {
                	try {
                		FileReader reader = new FileReader(chooser.getSelectedFile());
                		
                		// uloen 1. znaku - informace o velikosti okol
                		int okoli = Integer.parseInt(String.valueOf((char) reader.read()));
                		oknoAplikace.setVelikostOkoli(okoli);
                		pravidlo = new Pravidlo(okoli);
                		
                		int i;
                		int poc = 1;
                		while ((i = reader.read()) != -1) {
                			char znak = (char) i;
                			if (poc % (okoli + 1) == 0) {
                				if (znak == '1') {
                					pravidlo.zapisVysledek(true);
                				} else {
                					pravidlo.zapisVysledek(false);
                				}
                			}
                			poc++;
                		}
                		
                		reader.close();
                		
                		// zapsn vsledk do jednotlivch polek (mek):
                		for (int j = 0; j < pravidlo.getTabulkaVysledku().size(); j++) {
                			Bunka b = new Bunka();
                			if (pravidlo.getTabulkaVysledku().get(j) == true) {
                				b.setBarva(Color.WHITE);
                			} else {
                				b.setBarva(Color.BLACK);
                			}
                			mrizky.get(j).nastavBunkuAVykresliBunku(new Point(0, 0), b);
                		}
                		
                	} catch (IOException ioe) {
                		System.err.println(ioe);
                	}
                }
            	
            }
        });
        
        buttonZrusit.addActionListener(new java.awt.event.ActionListener() {
        	/**
        	 * Metoda pro zaven okna bez uloen i aplikovn zmn.
        	 */
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                zavriOkno();
            }
        });
        
        buttonReset.addActionListener(new java.awt.event.ActionListener() {
        	/**
        	 * Metoda pro obnoven vech pol na blou barvu.
        	 */
            public void actionPerformed(java.awt.event.ActionEvent evt) {
            	zavriOkno();
                oknoAplikace.vytvorOknoPravidlo();
            }
        });
        
        buttonAplikovat.addActionListener(new java.awt.event.ActionListener() {
        	/**
        	 * Metoda pro aplikaci zadanho pravidla na aktuln konfiguraci v mce.
        	 */
            public void actionPerformed(java.awt.event.ActionEvent evt) {
            	
            	// pidn aktulnch vsledk do pravidla:
            	List<Boolean> tabulka = new ArrayList<Boolean>();
            	for (int i = 0; i < tabulkaOkoli.size(); i++) {
	            	if (mrizky.get(i).getProstor()[0][0].getBarva() == Color.WHITE) {
	        			tabulka.add(true);
	        		} else {
	        			tabulka.add(false);
	        		}
            	}
            	pravidlo.setTabulkaVysledku(tabulka);
            	
            	// zjitn vech hodnot CA a zapsn do mky:
            	PocitacCA pocitac = new PocitacCA();
				List<Konfigurace> konfList;
				konfList = new ArrayList<Konfigurace>();
				konfList.add(new Konfigurace(oknoAplikace.getPocetAutomatuVRade()));
				boolean[] posloupnost = new boolean[oknoAplikace.getPocetAutomatuVRade()];
				for (int it = 0; it < oknoAplikace.getMrizka().getPocetSloupcu(); it++) {
					posloupnost[it] = oknoAplikace.getMrizka().getProstor()[it][0].x;
				}
				konfList.get(0).setPosloupnost(posloupnost);
				
				for (int in = 0; in < oknoAplikace.getPocetAutomatuVeSloupci(); in++) {
					konfList.add(new Konfigurace(oknoAplikace.getPocetAutomatuVRade()));
					konfList.set(in + 1, pocitac.vypocitejDalsiRadku(konfList.get(in), pravidlo));
				}
				
				oknoAplikace.getMrizka().setKonfigurace(konfList);
				oknoAplikace.getMrizka().repaint();
            }
        });
	}
	
	void zavriOkno() {
		dispose();
	}
}